// Notes: 
// * Make this as much hardcore C as possible because of MATLAB compiler issues
// What this entails:
// declare variables in front of the function! If you don't you will get bad errors!
// * REMEMBER: MATLAB HANDLES EVERYTHING AS DOUBLES!
// * REMEMBER: IND = y + x * image_height + c * (image_height*image_width);

// *****MUST HAVE:: include mex.h
#include "mex.h"
// you standard libraries por favors
#include "math.h"

// *****MUST HAVE:: argument macros
// 0 -> first argument on MATLAB, 1 -> second, 2-> etc...
// input buffer
#define	Image_In			prhs[0]
// output buffer
#define	Image_Out           prhs[1]
// input parameters
#define	Image_Height        prhs[2]
#define	Image_Width  	    prhs[3]
#define	Image_Channels  	prhs[4]
// padding parameters
#define	Start_Offset 	    prhs[5]
#define Column_Offset       prhs[6]
#define Color_Offset        prhs[7]
// filter parameters
#define Radius              prhs[8]

// DO WHATEVER HERE: 

void box_filter
        (
        double * input,
        double * output,
        int      height,
        int      width,
        int      column_offset,
        int      radius
        )
    {
    int x,y,i,j;
    double sum      ;
    double normalize;
    
    // NOTE: 
    // * since PADDING is involved, make sure the new indice calc is this
    // IND = y + x*column_offset
    // * since the pointer already accounted for offsets, it is OK to have negative indicies =D
    
    // REMEMBER: MATLAB operates in (y,x)- so this for look order
    // is more cache efficient:
    // ++x is more efficient than x++
    for (x = 0; x < width; ++x)
        for (y = 0; y < height; ++y)
            {
            sum         = 0.0f  ;
            normalize   = 0.0f  ;
            
            for (i = x-radius; i < x+radius+1; ++i)
                for (j = y-radius; j < y+radius+1; ++j)
                    {
                    sum         += input[j + i * column_offset] ;
                    normalize   += 1.0f                         ;
                    }
            
            output[y + x * column_offset] = sum/normalize;
            }
    }

// *****MUST HAVE:: include mex.h
void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray*prhs[])

	{ 
    // pointers to arguments
	double *image_in, *image_out, *image_height, *image_width, *image_channels, *start_offset, *column_offset, *color_offset, *radius;            
    int channel;
    
    /* Check for proper number of arguments */
	if (nrhs != 9)  
		mexErrMsgTxt("9 input arguments required: box_filter(image_in, image_out, image_height, image_width, image_channels, start_offset, column_offset, color_offset, radius"); 
	else if (nlhs > 1)
		mexErrMsgTxt("Too many output arguments."); 
   
	/* Assign pointers to the macros (look above) */ 
    // REMEMBER: MATLAB HANDLES EVERYTHING AS DOUBLES!
	image_in                = (double *) mxGetPr(Image_In);
	image_out               = (double *) mxGetPr(Image_Out);
	image_height            = (double *) mxGetPr(Image_Height);
	image_width             = (double *) mxGetPr(Image_Width);
    image_channels          = (double *) mxGetPr(Image_Channels);
	start_offset            = (double *) mxGetPr(Start_Offset);
    column_offset           = (double *) mxGetPr(Column_Offset);
    color_offset            = (double *) mxGetPr(Color_Offset);
    radius                  = (double *) mxGetPr(Radius);

	/* Do the actual computations in a subroutine */
    // In a filter, I usually handle one channel at a time for simplicity of writing the subroutine
    // REMEMBER TO APPLY START OFFSET TO POINTERS!
	
    // [R G B] is channel = [0 1 2]
    for (channel = 0; channel < (int) *image_channels; ++channel)
        {
        box_filter
        	(
        	image_in  + ((int) *start_offset) + ((int) *color_offset)*channel,
        	image_out + ((int) *start_offset) + ((int) *color_offset)*channel,
        	(int) *image_height,
        	(int) *image_width,
            (int) *column_offset,
            (int) *radius
            )
            ;
        // You can print out onto mex using this (great for debugging)
        mexPrintf("%d channel finished!\n", channel);
        }
    
    // that is it!
	return;
	}