%% Hello!
%  REQUIREMENTS:
%  1. C compiler
%  2. image processing toolbox
%  RECOMMENDED
%  Use 32 bit- 64 bit on older versions is still... crappy for mex (hard to
%  find compilers)

%  Step 1: goto Required folder
%  Step 2: type "mex -setup" and select the compiler you want (use default)
%  Step 3: type "mex box_blur.c" then check if some .mexBLAHBLAH pops out!
%  Step 4: goto root folder again
%  Step 5: type "tutorial" to run this tutorial
%  * READ COMMENTS through this
%  * READ COMMENTS through box_blur.c
%  * you have to recompile (using mex box_blur.c) everytime you modify
%  box_blur.c
%  & box_blur.c requires you to call box_blur(...) to call it



%% Housekeeping
% delete existing memory :: helps prevent memory overflow, etc.
clear all;
% adds these folders- so your "tutorial" function can use files within
% these folders
addpath('input');
addpath('required');

%% Parameters
% for square blur

blur_radius = 10;

%% Reading Images and Padding
% *********** READING IMAGE
% you can just do the file name 
% because you already added 'input' into path just make sure you have the file!
input_image = imread('1.png')           ;
% this convert the image from [0,255] -> [0,1] 
input_image = im2double(input_image)    ;

% NOTE: 
% * In MATLAB- it iterates through row the column! Not like how we do
% it in C!
% * INDEXING: MATLAB treats image buffers with the indexing like this
% To get index (x,y,c) -> ind
% IND = y + x * image_height + c * (image_height*im_width)
% * REMEMBER: MATLAB uses 1:height and C uses 0:height-1!!!!

%input_image parameters:: REMEMBER MATLAB IS HEIGHT, WIDTH, CHANNEL!
image_height   = size(input_image,1);
image_width    = size(input_image,2);
image_channels = size(input_image,3);

% *********** PADDING IMAGE :: CALCULATE THESE CAREFULLY
% pad image :: we usually use reflect padding
% we usually want to pad with the maximum patch radius you will be working
% with, in this case: blur_radius. We pad image_pal_val on all 4 sides
image_pad_val   = blur_radius;
% padarray(A, padsize, padval)
image_input_pad = padarray(input_image, [image_pad_val image_pad_val], 'symmetric');
% Now we want to know how to get to the next "column" which is height of
% padded
column_offset   = size(image_input_pad,1);
% start_offset :: we want to begin processing here not the beginning of the
% image because of padding
% We want to start at IND = (image_pad_val,image_pad_val) 
% = image_pad_val + image_pad_val*column_offset
% :: Remember, this is in image_pad (where width = column_offset)
start_offset    = image_pad_val + image_pad_val*column_offset;
% for color: remember to use padded height and width
color_offset    = size(image_input_pad,1) * size(image_input_pad,2);

%% Image Processing
% Make sure you built the mex file already or this function aint going to
% work!

% PREALLOCATE OUTPUT ::
% 1. you MUST preallocate
% 2. you MUST have a size of the PADDED
image_output_pad    = zeros(size(image_input_pad));

% Do the function!
% tic, toc times the subroutine!
% box_filter(image_in, image_out, image_height, image_width, image_channels, start_offset, column_offset, color_offset, radius)
tic;
box_filter(image_input_pad, image_output_pad, image_height, image_width, image_channels, start_offset, column_offset, color_offset, blur_radius);
fprintf('subourtine took: %f seconds!\n', toc);

% unpad: REMEMBER: MATLAB uses 1:height and C uses 0:height-1!!!!
image_out = image_output_pad(image_pad_val+1:image_height-image_pad_val, image_pad_val+1:image_width-image_pad_val, :);

%% Output
% write image :: always write in PNG!

imwrite(image_out, 'output/out.png');

% display image
figure;
imshow(image_out);


